import json

from collections import defaultdict
from flask.ext.restful import abort

from app import app
from endpoints.api import resource, nickname, require_repo_read, RepositoryParamResource
from data import model
from util.cache import cache_control


store = app.config['STORAGE']


def image_view(image):
  extended_props = image
  if image.storage and image.storage.id:
    extended_props = image.storage

  command = extended_props.command
  return {
    'id': image.docker_image_id,
    'created': extended_props.created,
    'comment': extended_props.comment,
    'command': json.loads(command) if command else None,
    'ancestors': image.ancestors,
    'dbid': image.id,
    'size': extended_props.image_size,
  }


@resource('/v1/repository/<path:repository>/image/')
class RepositoryImageList(RepositoryParamResource):
  """ Resource for listing repository images. """
  @require_repo_read
  @nickname('listRepositoryImages')
  def get(self, namespace, repository):
    """ List the images for the specified repository. """
    all_images = model.get_repository_images(namespace, repository)
    all_tags = model.list_repository_tags(namespace, repository)

    tags_by_image_id = defaultdict(list)
    for tag in all_tags:
      tags_by_image_id[tag.image.docker_image_id].append(tag.name)


    def add_tags(image_json):
      image_json['tags'] = tags_by_image_id[image_json['id']]
      return image_json


    return {
      'images': [add_tags(image_view(image)) for image in all_images]
    }


@resource('/v1/repository/<path:repository>/image/<image_id>')
class RepositoryImage(RepositoryParamResource):
  """ Resource for handling repository images. """
  @require_repo_read
  @nickname('getImage')
  def get(self, namespace, repository, image_id):
    image = model.get_repo_image(namespace, repository, image_id)
    if not image:
      abort(404)

    return image_view(image)


@resource('/v1/repository/<path:repository>/image/<image_id>/changes')
class RepositoryImageChanges(RepositoryParamResource):
  """ Resource for handling repository image change lists. """

  @cache_control(max_age=60*60)  # Cache for one hour
  @require_repo_read
  @nickname('getImageChanges')
  def get(self, namespace, repository, image_id):
    image = model.get_repo_image(namespace, repository, image_id)

    if not image:
      abort(404)

    uuid = image.storage and image.storage.uuid
    diffs_path = store.image_file_diffs_path(namespace, repository, image_id, uuid)

    try:
      response_json = store.get_content(diffs_path)
      return response_json
    except IOError:
      abort(404)
